# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401


class ElementNotFoundError(RuntimeError): ...

class InternalError(RuntimeError): ...

class InvalidArgumentError(RuntimeError): ...

class PermissionDeniedError(RuntimeError): ...

class PlatformError(RuntimeError): ...

class TimeoutError(RuntimeError): ...

class UnsupportedOperationError(RuntimeError): ...

class UnsupportedPlatformError(RuntimeError): ...
